/**
 * @fileoverview テキスト変数を使って章扉を作成する
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
 * @requires llibGetBoundsOfPage.jsx
*/

/*
    Copyright:
    2016-2018 Kenshi Muto <kmuto@debian.org>
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2016 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。
----------------------------------------------------------------------
*/
#include "../libs/libGetBoundsOfPage.jsx"

/**
 * 扉のマスターページを適用し、テキスト変数を展開する
 * @param {Object} object パラメータオブジェクト
 * @param {Document} obj.document ドキュメントオブジェクト
 * @param {MasterSpread} obj.master 扉のマスターページ
 * @param {MasterSpread} obj.p2master 2ページ目のマスターページ(省略可)
 * @param {String} obj.overrideitem[] マスターページの中でオーバライドするオブジェクトの名前
 * @param {ParagraphStyle} obj.lasttextlinepstyle リード文最後の行に適用する段落スタイル(改ページによって狂うことがあるため。省略可)
 * @param {ParagraphStyle} obj.applystyle このスタイルを発見したら扉にする 
 * @param {boolean} obj.applysection 扉でセクションを切り替える
 * @type Nothing
*/
function makeTobira2(obj) {
  if (obj.applystyle) return makeTobira2style(obj); 
  // マスターページを1、2ページに適用
  obj.document.pages[0].appliedMaster = obj.master;
  obj.document.pages[0].removeOverride();
  if (obj.p2master != null && obj.document.pages.length > 1) {
    obj.document.pages[1].appliedMaster = obj.p2master;
  }

  if (obj.overrideitem != null && obj.overrideitem.length > 0) {
    // マスターページオブジェクトのオーバライド
    var items = obj.document.pages[0].masterPageItems;
    for (var i = 0; i < items.length; i++) {
      for (var j = 0; j < obj.overrideitem.length; j++) {
        if (items[i].name == obj.overrideitem[j]) {
          items[i].override(obj.document.pages[0]);
          var target = obj.document.pages[0].textFrames.item(obj.overrideitem[j]);
          for (var k = target.textVariableInstances.length - 1; k >= 0; k--) {
            // テキスト変数を展開
            target.textVariableInstances[k].convertToText();
          }
        }
      }
    }
  }

  // リード文末尾の段落スタイルを変更
  var tf = obj.document.pages[0].textFrames.item("main-content");
  if (tf.lines.length > 0 && obj.lasttextlinepstyle != null) {
    tf.lines[tf.lines.length - 1].appliedParagraphStyle = obj.lasttextlinepstyle;
  }
  // リード文領域のテキストフレームのサイズを変更
  tf.visibleBounds = getBoundsOfPage(obj.document, obj.document.pages[0]);
}

/**
 * 指定のスタイルを発見したら、そのページに扉のマスターページを適用し、テキスト変数を展開する
 * @param {Object} object パラメータオブジェクト
 * @param {Document} obj.document ドキュメントオブジェクト
 * @param {MasterSpread} obj.master 扉のマスターページ
 * @param {MasterSpread} obj.p2master 次のページのマスターページ(省略可)
 * @param {String} obj.overrideitem[] マスターページの中でオーバライドするオブジェクトの名前
 * @param {ParagraphStyle} obj.lasttextlinepstyle リード文最後の行に適用する段落スタイル(改ページによって狂うことがあるため。省略可)
 * @param {ParagraphStyle} obj.applystyle このスタイルを発見したら扉にする 
 * @param {boolean} obj.applysection 扉でセクションを切り替える
 * @type Nothing
*/
function makeTobira2style(obj) {
  // セクションを消す
  for (var i = obj.document.sections.length - 1; i >= 0; i--) {
    if (obj.document.sections[i].label.match(/\-tmp/)) obj.document.sections[i].remove();
  }
  var sec = 1;
    
  for (var p = obj.document.pages.length - 1; p >= 0; p--) { // いちおう後ろから加工
    var lines = obj.document.pages[p].textFrames.item("main-content").lines;
    if (lines.length < 2 || lines[0].appliedParagraphStyle != obj.applystyle) continue; // 1行だけの場合は改ページ白
    
    obj.document.pages[p].appliedMaster = obj.master;
    obj.document.pages[p].removeOverride();
    if (obj.applysection) {
      // タグがあるので先頭2つ飛ばす
      // name:lines[0].contents.replace("\r", "").slice(2, 6) + "-" を指定したらPDFにも全部ついてしまってよくない
      var sect = obj.document.sections.add(obj.document.pages[p], {label: "sec-tmp-" + sec, continueNumbering: true});
      sec += 1;
    }

    if (obj.p2master != null && (obj.document.pages.length - 1) > p) {
       obj.document.pages[p + 1].appliedMaster = obj.p2master;
    }

    if (obj.overrideitem != null && obj.overrideitem.length > 0) {
      // マスターページオブジェクトのオーバライド
      var items = obj.document.pages[p].masterPageItems;
      for (var i = 0; i < items.length; i++) {
        for (var j = 0; j < obj.overrideitem.length; j++) {
          if (items[i].name == obj.overrideitem[j]) {
            items[i].override(obj.document.pages[p]);
            var target = obj.document.pages[p].textFrames.item(obj.overrideitem[j]);
            for (var k = 0; k < target.textVariableInstances.length; k++) {
              // テキスト変数を展開
              target.textVariableInstances[k].convertToText();
            }
          }
        }
      }
    }

    // リード文末尾の段落スタイルを変更
    var tf = obj.document.pages[p].textFrames.item("main-content");
    if (tf.lines.length > 0 && obj.lasttextlinepstyle != null) {
      tf.lines[tf.lines.length - 1].appliedParagraphStyle = obj.lasttextlinepstyle;
    }
    // リード文領域のテキストフレームのサイズを変更
    tf.visibleBounds = getBoundsOfPage(obj.document, obj.document.pages[p]);
  }
}
